@@ -1,4 +1,5 @@
-'''An asynchronous multi-process `HTTP proxy server`_
+'''An asynchronous multi-process `HTTP proxy server`_. It works for both
+``http`` and ``https`` (tunneled) requests.
 
 
 Managing Headers
@@ -37,7 +38,9 @@ Implemenation
 '''
 import io
 import sys
+import logging
 from functools import partial
+from asyncio import Queue, QueueEmpty
 
 try:
     import pulsar
@@ -45,7 +48,7 @@ except ImportError:
     sys.path.append('../../')
     import pulsar
 
-from pulsar import HttpException, Queue, Empty, Deferred
+from pulsar import HttpException, async, coroutine_return, add_errback
 from pulsar.apps import wsgi, http
 from pulsar.utils.httpurl import Headers
 from pulsar.utils.log import LocalMixin, local_property
@@ -54,6 +57,7 @@ from pulsar.utils.log import LocalMixin, local_property
 SERVER_SOFTWARE = 'Pulsar-proxy-server/%s' % pulsar.version
 ENVIRON_HEADERS = ('content-type', 'content-length')
 USER_AGENT = SERVER_SOFTWARE
+logger = logging.getLogger('pulsar.proxyserver')
 
 
 def x_forwarded_for(environ, headers):
@@ -71,51 +75,55 @@ class user_agent:
 
 
 class ProxyServerWsgiHandler(LocalMixin):
-    '''WSGI middleware for an asynchronous proxy server. To perform
-processing on headers you can pass a list of ``headers_middleware``.
-An headers middleware is a callable which accepts two parameters, the wsgi
-*environ* dictionary and the *headers* container.'''
+    '''WSGI middleware for an asynchronous proxy server.
+
+    To perform processing on headers you can pass a list of
+    ``headers_middleware``.
+    An headers middleware is a callable which accepts two parameters, the wsgi
+    *environ* dictionary and the *headers* container.
+    '''
     def __init__(self, headers_middleware=None):
         self.headers_middleware = headers_middleware or []
 
     @local_property
     def http_client(self):
-        '''The :ref:`HttpClient <'''
+        '''The :class:`.HttpClient` used by this proxy middleware for
+        accessing upstream resources'''
         return http.HttpClient(decompress=False, store_cookies=False)
 
     def __call__(self, environ, start_response):
         # The WSGI thing
-        if environ.get('HTTP_EXPECT') == '100-continue':
-            return self._call(environ, start_response)
-        stream = environ.get('wsgi.input') or io.BytesIO()
-        data = stream.read()
-        # handle asynchronous body data
-        if isinstance(data, Deferred):
-            return data.add_callback(partial(self._call, environ,
-                                             start_response))
-        else:
-            return self._call(environ, start_response, data)
+        loop = environ['pulsar.connection']._loop
+        return async(self._call(environ, start_response, loop), loop)
 
-    def _call(self, environ, start_response, data=None):
+    def _call(self, environ, start_response, loop):
         uri = environ['RAW_URI']
+        logger.debug('new request for %r' % uri)
         if not uri or uri.startswith('/'):  # No proper uri, raise 404
             raise HttpException(status=404)
+        if environ.get('HTTP_EXPECT') != '100-continue':
+            stream = environ.get('wsgi.input') or io.BytesIO()
+            data = yield stream.read()
+        else:
+            data = None
         request_headers = self.request_headers(environ)
         method = environ['REQUEST_METHOD']
-        response = self.http_client.request(method, uri,
-                                            data=data,
-                                            headers=request_headers,
-                                            version=environ['SERVER_PROTOCOL'])
-        #
+
         if method == 'CONNECT':
-            return ProxyTunnel(environ, start_response, response)
+            response = ProxyTunnel(environ, start_response)
         else:
-            return ProxyResponse(environ, start_response, response)
+            response = ProxyResponse(environ, start_response)
+        request = self.http_client.request(method, uri, data=data,
+                                           headers=request_headers,
+                                           version=environ['SERVER_PROTOCOL'],
+                                           pre_request=response.pre_request)
+        add_errback(request, response.error)
+        coroutine_return(response)
 
     def request_headers(self, environ):
         '''Fill request headers from the environ dictionary and
-modify the headers via the list of :attr:`headers_middleware`.
-The returned headers will be sent to the target uri.'''
+        modify the headers via the list of :attr:`headers_middleware`.
+        The returned headers will be sent to the target uri.'''
         headers = Headers(kind='client')
         for k in environ:
             if k.startswith('HTTP_'):
@@ -131,30 +139,58 @@ The returned headers will be sent to the target uri.'''
         return headers
 
 
+############################################################################
+##    RESPONSE OBJECTS
 class ProxyResponse(object):
     '''Asynchronous wsgi response.
     '''
+    _started = False
     _headers = None
     _done = False
 
-    def __init__(self, environ, start_response, response):
+    def __init__(self, environ, start_response):
+        self._loop = environ['pulsar.connection']._loop
         self.environ = environ
         self.start_response = start_response
         self.queue = Queue()
-        response.on_finished.add_errback(self.error)
-        self.setup(response)
-
-    def setup(self, response):
-        response.bind_event('data_processed', self.data_processed)
 
     def __iter__(self):
         while True:
-            try:
-                yield self.queue.get(wait=not self._done)
-            except Empty:
-                break
+            if self._done:
+                try:
+                    yield self.queue.get_nowait()
+                except QueueEmpty:
+                    break
+            else:
+                yield async(self.queue.get(), loop=self._loop)
 
-    def data_processed(self, response, **kw):
+    def pre_request(self, response, exc=None):
+        self._started = True
+        response.bind_event('data_processed', self.data_processed)
+        return response
+
+    def error(self, exc):
+        if not self._started:
+            request = wsgi.WsgiRequest(self.environ)
+            content_type = request.content_types.best_match(
+                ('text/html', 'text/plain'))
+            uri = self.environ['RAW_URI']
+            msg = 'Could not find %s' % uri
+            logger.info(msg=msg)
+            if content_type == 'text/html':
+                html = wsgi.HtmlDocument(title=msg)
+                html.body.append('<h1>%s</h1>' % msg)
+                data = html.render()
+                resp = wsgi.WsgiResponse(504, data, content_type='text/html')
+            elif content_type == 'text/plain':
+                resp = wsgi.WsgiResponse(504, msg, content_type='text/html')
+            else:
+                resp = wsgi.WsgiResponse(504, '')
+            self.start_response(resp.status, resp.get_headers())
+            self._done = True
+            self.queue.put_nowait(resp.content[0])
+
+    def data_processed(self, response, exc=None, **kw):
         '''Receive data from the requesting HTTP client.'''
         status = response.get_status()
         if status == '100 Continue':
@@ -170,21 +206,7 @@ class ProxyResponse(object):
             body = response.recv_body()
             if response.parser.is_message_complete():
                 self._done = True
-            self.queue.put(body)
-
-    def error(self, failure):
-        '''Handle a failure.'''
-        if not self._done:
-            uri = self.environ['RAW_URI']
-            msg = 'Oops! Could not find %s' % uri
-            html = wsgi.HtmlDocument(title=msg)
-            html.body.append('<h1>%s</h1>' % msg)
-            data = html.render()
-            resp = wsgi.WsgiResponse(504, data, content_type='text/html')
-            self.start_response(resp.status, resp.get_headers(),
-                                failure.exc_info)
-            self._done = True
-            self.queue.put(resp.content[0])
+            self.queue.put_nowait(body)
 
     def remove_hop_headers(self, headers):
         for header, value in headers:
@@ -194,75 +216,69 @@ class ProxyResponse(object):
 
 class ProxyTunnel(ProxyResponse):
 
-    def setup(self, response):
-        response.bind_event('pre_request', self.connection_made)
-
-    def connection_made(self, response):
+    def pre_request(self, response, exc=None):
         '''Start the tunnel.
 
-        This is a callback fired once a connection with target server is
-        established and this proxy is acting as tunnel for a TSL server.
+        This is a callback fired once a connection with upstream server is
+        established.
 
         Write back to the client the 200 Connection established message.
         After this the downstream connection consumer will upgrade to the
         DownStreamTunnel.
         '''
         # Upgrade downstream protocol consumer
-        upstream = response.connection
-        downstream = self.environ['pulsar.connection']
-        downstream.upgrade(partial(DownStreamTunnel, upstream))
+        # set the request to None so that start_request is not called
+        assert response._request.method == 'CONNECT'
+        self._started = True
+        response._request = None
+        upstream = response._connection
+        dostream = self.environ['pulsar.connection']
+        #
+        dostream.upgrade(partial(StreamTunnel, upstream))
+        upstream.upgrade(partial(StreamTunnel, dostream))
+        response.finished()
         self.start_response('200 Connection established', [])
-        self._done = True
         # send empty byte so that headers are sent
-        self.queue.put(b'')
+        self.queue.put_nowait(b'')
+        self._done = True
         return response
 
 
-class DownStreamTunnel(pulsar.ProtocolConsumer):
+class StreamTunnel(pulsar.ProtocolConsumer):
     ''':class:`ProtocolConsumer` handling encrypted messages from
-    downstream client.
+    downstream client and upstream server.
 
     This consumer is created as an upgrade of the standard Http protocol
-    consumer, once encrypted data arrives from the downstream client.
+    consumer.
 
-    .. attribute:: upstream
+    .. attribute:: tunnel
 
-        Client :class:`pulsar.Connection` with the upstream server.
+        Connection to the downstream client or upstream server.
     '''
-    def __init__(self, upstream):
-        super(DownStreamTunnel, self).__init__()
-        self.upstream = upstream
-
-    def connection_made(self, connection):
-        '''Upgrade the consumer of :attr:`upstream` connection.
-
-        The upstream (proxy - endpoint) connection consumer changes from HTTP
-        to the :class:`UpstreamTunnel`.
-        '''
-        self.upstream.upgrade(partial(UpstreamTunnel, connection), True)
-
-    def data_received(self, data):
-        # Received data from the downstream part of the tunnel.
-        # Send the data to the upstream server
-        self.upstream.transport.write(data)
-
-
-class UpstreamTunnel(pulsar.ProtocolConsumer):
     headers = None
     status_code = None
 
-    def __init__(self, downstream):
-        super(UpstreamTunnel, self).__init__()
-        self.downstream = downstream
+    def __init__(self, tunnel):
+        super(StreamTunnel, self).__init__()
+        self.tunnel = tunnel
+
+    def connection_made(self, connection):
+        connection.bind_event('connection_lost', self._close_tunnel)
 
     def data_received(self, data):
-        # Got data from the upstream server.
-        # Send it back to the downstream client
-        self.downstream.transport.write(data)
+        try:
+            self.tunnel._transport.write(data)
+        except Exception:
+            if not self.tunnel.closed:
+                raise
+
+    def _close_tunnel(self, arg, exc=None):
+        if not self.tunnel.closed:
+            self.tunnel._loop.call_soon(self.tunnel.close)
 
 
-def server(name='proxy-server', headers_middleware=None, server_software=None,
-           **kwargs):
+def server(name='proxy-server', headers_middleware=None,
+           server_software=None, **kwargs):
     '''Function to Create a WSGI Proxy Server.'''
     if headers_middleware is None:
         #headers_middleware = [user_agent(USER_AGENT), x_forwarded_for]
