@@ -1,7 +1,7 @@
 '''Asynchronous application for serving requests
-on sockets. This is the base class of :class:`pulsar.apps.wsgi.WSGIServer`.
+on sockets. This is the base class of :class:`.WSGIServer`.
 All is needed by a :class:`SocketServer` is a callable which build a
-:class:`pulsar.ProtocolConsumer` for each new client request received.
+:class:`.ProtocolConsumer` for each new client request received.
 This is an example of a script for an Echo server::
 
     import pulsar
@@ -14,7 +14,7 @@ This is an example of a script for an Echo server::
         SocketServer(EchoServerProtocol).start()
 
 Check the :ref:`echo server example <tutorials-writing-clients>` for detailed
-implementation of the ``EchoServerProtocol`` class.
+implementation of the ``.EchoServerProtocol`` class.
 
 .. _socket-server-settings:
 
@@ -45,7 +45,7 @@ rarely used.
 
 keep_alive
 ---------------
-To control how long a server :class:`pulsar.Connection` is kept alive after the
+To control how long a server :class:`.Connection` is kept alive after the
 last read from the remote client, one can use the
 :ref:`keep-alive <setting-keep_alive>` setting::
 
@@ -79,9 +79,9 @@ When running a :class:`SocketServer` in threading mode::
 
     python script.py --concurrency thread
 
-the number of :class:`pulsar.Actor` serving the application is set
+the number of :class:`.Actor` serving the application is set
 to ``0`` so that the application is actually served by the
-:class:`pulsar.Arbiter` event-loop (we refer this to a single process server).
+:class:`.Arbiter` event-loop (we refer this to a single process server).
 This configuration is used when debugging, testing, benchmarking or on small
 load servers.
 
@@ -95,10 +95,14 @@ for:
 Check the :meth:`SocketServer.monitor_start` method for implementation details.
 '''
 import os
+import socket
+from math import log
+from random import lognormvariate
 from functools import partial
+from asyncio import DatagramProtocol, Protocol
 
 import pulsar
-from pulsar import TcpServer, multi_async
+from pulsar import TcpServer, DatagramServer, Connection
 from pulsar.utils.internet import (parse_address, SSLContext, WrapSocket,
                                    format_address)
 from pulsar.utils.config import pass_through
@@ -171,8 +175,30 @@ class CertFile(SocketSetting):
     """
 
 
+class WrapTransport:
+
+    def __init__(self, transport):
+        self.extra = transport._extra
+        self.sock = self.extra.pop('socket')
+        self.transport = transport.__class__
+
+    def __getstate__(self):
+        s = self.sock
+        d = self.__dict__.copy()
+        d['sock'] = (s.fileno(), s.family, s.type, s.proto)
+        return d
+
+    def __setstate__(self, state):
+        values = state.pop('sock')
+        self.__dict__ = state
+        self.sock = socket.fromfd(*values)
+
+    def __call__(self, loop, protocol):
+        return self.transport(loop, self.sock, protocol, extra=self.extra)
+
+
 class SocketServer(pulsar.Application):
-    '''A :class:`pulsar.apps.Application` which serve application on a socket.
+    '''A :class:`.Application` which serve application on a socket.
 
     It bind a socket to a given address and listen for requests. The request
     handler is constructed from the callable passed during initialisation.
@@ -182,15 +208,14 @@ class SocketServer(pulsar.Application):
         The socket address, available once the application has started.
     '''
     name = 'socket'
-    address = None
     cfg = pulsar.Config(apps=['socket'])
 
-    def protocol_consumer(self):
-        '''Factory of :class:`pulsar.ProtocolConsumer` used by the server.
+    def protocol_factory(self):
+        '''Factory of :class:`.ProtocolConsumer` used by the server.
 
-        By default it returns the :attr:`pulsar.apps.Application.callable`
-        attribute.'''
-        return self.callable
+        By default it returns the :meth:`.Application.callable`.
+        '''
+        return partial(Connection, self.cfg.callable)
 
     def monitor_start(self, monitor):
         '''Create the socket listening to the ``bind`` address.
@@ -199,7 +224,7 @@ class SocketServer(pulsar.Application):
         number of workers to 0.
         '''
         cfg = self.cfg
-        loop = monitor.event_loop
+        loop = monitor._loop
         if (not pulsar.platform.has_multiProcessSocket
                 or cfg.concurrency == 'thread'):
             cfg.set('workers', 0)
@@ -216,61 +241,137 @@ class SocketServer(pulsar.Application):
             ssl = SSLContext(keyfile=cfg.key_file, certfile=cfg.cert_file)
         address = parse_address(self.cfg.address)
         # First create the sockets
-        sockets = yield loop.start_serving(lambda: None, *address)
+        server = yield loop.create_server(Protocol, *address)
         addresses = []
-        for sock in sockets:
-            assert loop.remove_reader(sock.fileno()), (
-                "Could not remove reader")
+        sockets = []
+        for sock in server.sockets:
             addresses.append(sock.getsockname())
-        monitor.params.sockets = [WrapSocket(s) for s in sockets]
-        monitor.params.ssl = ssl
-        self.addresses = addresses
-        self.address = addresses[0]
+            sockets.append(WrapSocket(sock))
+            server.loop.remove_reader(sock.fileno())
+        monitor.sockets = sockets
+        monitor.ssl = ssl
+        cfg.addresses = addresses
 
-    def worker_start(self, worker):
-        '''Start the worker by invoking the :meth:`create_server` method.'''
-        worker.servers[self.name] = servers = []
-        for sock in worker.params.sockets:
-            server = self.create_server(worker, sock.sock)
-            server.bind_event('stop', partial(self._stop_worker, worker))
-            servers.append(server)
+    def actorparams(self, monitor, params):
+        params.update({'sockets': monitor.sockets, 'ssl': monitor.ssl})
 
-    def worker_stopping(self, worker):
-        all = []
-        for server in worker.servers[self.name]:
-            all.append(server.close_connections())
-        return multi_async(all)
+    def worker_start(self, worker, exc=None):
+        '''Start the worker by invoking the :meth:`create_server` method.
+        '''
+        if not exc:
+            server = self.create_server(worker)
+            server.bind_event('stop', lambda _, **kw: worker.stop())
+            worker.servers[self.name] = server
+
+    def worker_stopping(self, worker, exc=None):
+        server = worker.servers.get(self.name)
+        if server:
+            return server.close()
 
     def worker_info(self, worker, info):
-        info['sockets'] = sockets = []
-        for server in worker.servers.get(self.name, ()):
-            address = format_address(server.address)
-            sockets.append({
-                'address': format_address(server.address),
-                'read_timeout': server.timeout,
-                'concurrent_connections': server.concurrent_connections,
-                'received_connections': server.received})
+        server = worker.servers.get(self.name)
+        if server:
+            info['%sserver' % self.name] = server.info()
+        return info
 
-    def _stop_worker(self, worker, exc):
-        worker.stop()
-        return exc
+    def server_factory(self, *args, **kw):
+        '''Create a :class:`.TcpServer`.
+        '''
+        return TcpServer(*args, **kw)
 
     #   INTERNALS
+    def create_server(self, worker):
+        '''Create the Server which will listen for requests.
 
-    def create_server(self, worker, sock, ssl=None):
-        '''Create the Server Protocol which will listen for requests. It
-uses the :meth:`protocol_consumer` method as the protocol consumer factory.'''
+        :return: a :class:`.TcpServer`.
+        '''
+        sockets = [sock.sock for sock in worker.sockets]
         cfg = self.cfg
-        server = TcpServer(worker.event_loop,
-                           sock=sock,
-                           consumer_factory=self.protocol_consumer(),
-                           max_connections=cfg.max_requests,
-                           timeout=cfg.keep_alive,
-                           name=self.name)
+        max_requests = cfg.max_requests
+        if max_requests:
+            max_requests = int(lognormvariate(log(max_requests), 0.2))
+        server = self.server_factory(self.protocol_factory(),
+                                     worker._loop,
+                                     sockets=sockets,
+                                     max_connections=max_requests,
+                                     keep_alive=cfg.keep_alive,
+                                     name=self.name)
         for event in ('connection_made', 'pre_request', 'post_request',
                       'connection_lost'):
             callback = getattr(cfg, event)
             if callback != pass_through:
                 server.bind_event(event, callback)
-        server.start_serving(cfg.backlog, sslcontext=worker.params.ssl)
+        server.start_serving(cfg.backlog, sslcontext=worker.ssl)
+        return server
+
+
+class UdpSocketServer(SocketServer):
+    '''A :class:`.SocketServer` which serves application on a UDP sockets.
+
+    It binds a socket to a given address and listen for requests. The request
+    handler is constructed from the callable passed during initialisation.
+
+    .. attribute:: address
+
+        The socket address, available once the application has started.
+    '''
+    name = 'udpsocket'
+    cfg = pulsar.Config(apps=['socket'])
+
+    def protocol_factory(self):
+        '''Return the :class:`.DatagramProtocol` factory.
+        '''
+        return self.cfg.callable
+
+    def monitor_start(self, monitor):
+        '''Create the socket listening to the ``bind`` address.
+
+        If the platform does not support multiprocessing sockets set the
+        number of workers to 0.
+        '''
+        cfg = self.cfg
+        loop = monitor._loop
+        if (not pulsar.platform.has_multiProcessSocket
+                or cfg.concurrency == 'thread'):
+            cfg.set('workers', 0)
+        if not cfg.address:
+            raise pulsar.ImproperlyConfigured('Could not open a socket. '
+                                              'No address to bind to')
+        address = parse_address(self.cfg.address)
+        # First create the sockets
+        t, _ = yield loop.create_datagram_endpoint(DatagramProtocol, address)
+        sock = t.get_extra_info('socket')
+        assert loop.remove_reader(sock.fileno())
+        monitor.sockets = [WrapTransport(t)]
+        cfg.addresses = [sock.getsockname()]
+
+    def actorparams(self, monitor, params):
+        params.update({'sockets': monitor.sockets})
+
+    def server_factory(self, *args, **kw):
+        '''By default returns a new :class:`.DatagramServer`.
+        '''
+        return DatagramServer(*args, **kw)
+
+    #   INTERNALS
+    def create_server(self, worker):
+        '''Create the Server which will listen for requests.
+
+        :return: the server obtained from :meth:`server_factory`.
+        '''
+        cfg = self.cfg
+        max_requests = cfg.max_requests
+        if max_requests:
+            max_requests = int(lognormvariate(log(max_requests), 0.2))
+        server = self.server_factory(self.protocol_factory(),
+                                     worker._loop,
+                                     sockets=worker.sockets,
+                                     max_requests=max_requests,
+                                     name=self.name)
+        server.bind_event('stop', lambda _, **kw: worker.stop())
+        for event in ('pre_request', 'post_request'):
+            callback = getattr(cfg, event)
+            if callback != pass_through:
+                server.bind_event(event, callback)
+        server.create_endpoint()
         return server
