@@ -2,8 +2,8 @@
 handling the WebSocket_ protocol.
 Web sockets allow for bidirectional communication between the browser
 and server. Pulsar implementation uses the WSGI middleware
-:class:`WebSocket` for the handshake_ and a class derived from
-:class:`WS` handler for the communication part.
+:class:`.WebSocket` for the handshake_ and a class derived from
+:class:`.WS` handler for the communication part.
 
 This is a Web Socket handler which echos all received messages
 back to the client::
@@ -16,7 +16,7 @@ back to the client::
             websocket.write(message)
 
 
-To create a valid :class:`WebSocket` middleware initialise as follow::
+To create a valid :class:`.WebSocket` middleware initialise as follow::
 
     wm = ws.WebSocket('/bla', EchoWS())
     app = wsgi.WsgiHandler(middleware=(..., wm))
@@ -32,14 +32,6 @@ API
 
 .. _websocket-middleware:
 
-WebSocket
-~~~~~~~~~~~~~~~~
-
-.. autoclass:: WebSocket
-   :members:
-   :member-order: bysource
-
-
 .. _websocket-handler:
 
 WebSocket handler
@@ -49,6 +41,14 @@ WebSocket handler
    :members:
    :member-order: bysource
 
+.. module:: pulsar.apps.ws.websocket
+
+WebSocket
+~~~~~~~~~~~~~~~~
+
+.. autoclass:: WebSocket
+   :members:
+   :member-order: bysource
 
 WebSocket protocol
 ~~~~~~~~~~~~~~~~~~~~
@@ -65,9 +65,9 @@ class WS(object):
     '''A web socket handler for both servers and clients.
 
     It implements the asynchronous message passing for a
-    :class:`WebSocketProtocol`.
+    :class:`.WebSocketProtocol`.
     On the server, the communication is started by the
-    :class:`WebSocket` middleware after a successful handshake.
+    :class:`.WebSocket` middleware after a successful handshake.
 
     Override :meth:`on_message` to handle incoming string messages and
     :meth:`on_bytes` to handle incoming ``bytes`` messages.
@@ -76,16 +76,20 @@ class WS(object):
     specific tasks when the websocket is opened or closed.
 
     These methods accept as first parameter the
-    :class:`WebSocketProtocol` created during the handshake.
+    :class:`.WebSocketProtocol` created during the handshake.
     '''
     def on_open(self, websocket):
-        """Invoked when a new WebSocket is opened."""
+        '''Invoked when a new ``websocket`` is opened.
+
+        A web socket is opened straight after the upgrade headers are
+        sent (servers) or received (clients).
+        '''
         pass
 
     def on_message(self, websocket, message):
         '''Handles incoming messages on the WebSocket.
 
-        This method must be overloaded.
+        This method should be overwritten
         '''
         pass
 
@@ -93,22 +97,18 @@ class WS(object):
         '''Handles incoming bytes.'''
         pass
 
-    def on_ping(self, websocket, body):
-        '''Handle incoming ping ``Frame``.'''
-        websocket.pong(body)
+    def on_ping(self, websocket, message):
+        '''Handle incoming ping ``message``.
+
+        By default it writes back the message as a ``pong`` frame.
+        '''
+        websocket.pong(message)
 
     def on_pong(self, websocket, body):
-        '''Handle incoming pong ``Frame``.'''
+        '''Handle incoming pong ``message``.'''
         pass
 
     def on_close(self, websocket):
-        """Invoked when the WebSocket is closed."""
+        """Invoked when the ``websocket`` is closed.
+        """
         pass
-
-    def pong(self, websocket, body=None):
-        '''Return a ``pong`` frame.'''
-        return websocket.parser.pong(body)
-
-    def ping(self, websocket, body=None):
-        '''Return a ``ping`` frame.'''
-        return websocket.parser.ping(body)
