@@ -10,16 +10,16 @@ from pulsar import HaltServer
 
 from .actor import Actor, ACTOR_STATES
 from .monitor import PoolMixin, Monitor, _spawn_actor
-from .defer import multi_async
+from .futures import multi_async
 from .access import get_actor, set_actor
-from . import proxy
+from .proxy import actor_proxy_future
 
 
 __all__ = ['arbiter', 'spawn', 'Arbiter']
 
 
-def arbiter(commands_set=None, **params):
-    '''Obtain the :class:`Arbiter`.
+def arbiter(**params):
+    '''Obtain the :class:`.Arbiter`.
 
     It returns the arbiter instance only if we are on the arbiter
     context domain, otherwise it returns nothing.
@@ -32,13 +32,12 @@ def arbiter(commands_set=None, **params):
         return arbiter
 
 
-# TODO: why cfg is set to None?
-def spawn(cfg=None, **kwargs):
-    '''Spawn a new :class:`Actor` and return an :class:`ActorProxyDeferred`.
+def spawn(**kwargs):
+    '''Spawn a new :class:`.Actor` and return an :class:`.ActorProxyFuture`.
 
-    This method can be used from any :class:`Actor`.
-    If not in the :class:`Arbiter` domain, the method sends a request
-    to the :class:`Arbiter` to spawn a new actor.
+    This method can be used from any :class:`.Actor`.
+    If not in the :class:`.Arbiter` domain, the method sends a request
+    to the :class:`.Arbiter` to spawn a new actor.
     Once the arbiter creates the actor it returns the ``proxy`` to the
     original caller.
 
@@ -46,13 +45,13 @@ def spawn(cfg=None, **kwargs):
 
     These optional parameters are:
 
-    * ``actor_class`` a custom :class:`Actor` subclass
     * ``aid`` the actor id
     * ``name`` the actor name
     * :ref:`actor hooks <actor-hooks>` such as ``start``, ``stopping``
       and ``stop``
+    * ``actor_class`` a custom :class:`.Actor` subclass (never used)
 
-    :return: an :class:`ActorProxyDeferred`.
+    :return: an :class:`.ActorProxyFuture`.
 
     A typical usage::
 
@@ -63,7 +62,7 @@ def spawn(cfg=None, **kwargs):
         'ba42b02b'
         >>> a.called
         True
-        >>> p = a.result
+        >>> p = a.result()
         >>> p.address
         ('127.0.0.1', 46691)
     '''
@@ -74,13 +73,13 @@ def spawn(cfg=None, **kwargs):
     # We send a message to the Arbiter to spawn a new Actor
     if not isinstance(actor, Arbiter):
         # send the request to the arbiter
-        msg = actor.send('arbiter', 'spawn', **kwargs)
-        return proxy.ActorProxyDeferred(aid, msg)
+        future = actor.send('arbiter', 'spawn', **kwargs)
+        return actor_proxy_future(aid, future)
     else:
         return actor.spawn(**kwargs)
 
 
-def stop_arbiter(self):     # pragma    nocover
+def stop_arbiter(self, exc=None):     # pragma    nocover
     p = self.pidfile
     if p is not None:
         self.logger.debug('Removing %s' % p.fname)
@@ -100,10 +99,9 @@ def stop_arbiter(self):     # pragma    nocover
     return self
 
 
-def start_arbiter(self):
+def start_arbiter(self, exc=None):
     if current_process().daemon:
-        raise pulsar.PulsarException(
-            'Cannot create the arbiter in a daemon process')
+        raise HaltServer('Cannot create the arbiter in a daemon process')
     os.environ["SERVER_SOFTWARE"] = pulsar.SERVER_SOFTWARE
     pidfile = self.cfg.pidfile
     if pidfile is not None:
@@ -113,7 +111,6 @@ def start_arbiter(self):
         except RuntimeError as e:
             raise HaltServer('ERROR. %s' % str(e), exit_code=3)
         self.pidfile = p
-    return self
 
 
 def info_arbiter(self, info=None):
@@ -142,12 +139,11 @@ def info_arbiter(self, info=None):
 class Arbiter(PoolMixin):
     '''The Arbiter drives pulsar servers.
 
-    It is the most important a :class:`Actor` and :class:`PoolMixin` in
+    It is the most important a :class:`.Actor` and :class:`.PoolMixin` in
     pulsar concurrent framework. It is used as singleton
-    in the main process and it manages one or more :class:`Monitor`.
-    It runs the main :class:`EventLoop` of your concurrent application.
-    It is the equivalent of the gunicorn_ arbiter, the twisted_ reactor
-    and the tornado_ eventloop.
+    in the main process and it manages one or more :class:`.Monitor`.
+    It runs the main :ref:`event loop <asyncio-event-loop>` of your
+    concurrent application.
 
     Users access the arbiter (in the arbiter process domain) by the
     high level api::
@@ -155,10 +151,6 @@ class Arbiter(PoolMixin):
         import pulsar
 
         arbiter = pulsar.arbiter()
-
-    .. _gunicorn: http://gunicorn.org/
-    .. _twisted: http://twistedmatrix.com/trac/
-    .. _tornado: http://www.tornadoweb.org/
     '''
     pidfile = None
 
@@ -174,12 +166,12 @@ class Arbiter(PoolMixin):
     # ARBITER HIGH LEVEL API
     ########################################################################
     def add_monitor(self, monitor_name, monitor_class=None, **params):
-        '''Add a new :class:`Monitor` to the :class:`Arbiter`.
+        '''Add a new :class:`.Monitor` to the :class:`Arbiter`.
 
-        :param monitor_class: a :class:`pulsar.Monitor` class.
+        :param monitor_class: a :class:`.Monitor` class.
         :param monitor_name: a unique name for the monitor.
         :param kwargs: dictionary of key-valued parameters for the monitor.
-        :return: the :class:`pulsar.Monitor` added.
+        :return: the :class:`.Monitor` added.
         '''
         if monitor_name in self.registered:
             raise KeyError('Monitor "%s" already available' % monitor_name)
@@ -192,7 +184,7 @@ class Arbiter(PoolMixin):
         return m
 
     def close_monitors(self):
-        '''Close all :class:`Monitor` at once.
+        '''Close all :class:`.Monitor` at once.
         '''
         return multi_async((m.stop() for m in list(itervalues(self.monitors))))
 
@@ -220,6 +212,10 @@ class Arbiter(PoolMixin):
     # INTERNALS
     ########################################################################
     def _remove_actor(self, actor, log=True):
-        super(Arbiter, self)._remove_actor(actor, log)
-        self.registered.pop(actor.name, None)
-        self.monitors.pop(actor.aid, None)
+        a = super(Arbiter, self)._remove_actor(actor, False)
+        b = self.registered.pop(actor.name, None)
+        c = self.monitors.pop(actor.aid, None)
+        removed = a or b or c
+        if removed and log:
+            self.logger.warning('Removing %s', actor)
+        return removed
