@@ -1,57 +1,71 @@
 '''Actors communicate with each other by sending and receiving messages.
 The :mod:`pulsar.async.mailbox` module implements the message passing layer
-via a bidirectional socket connections between the :class:`pulsar.Arbiter`
-and :class:`pulsar.Actor`.
+via a bidirectional socket connections between the :class:`.Arbiter`
+and any :class:`.Actor`.
 
 Message sending is asynchronous and safe, the message is guaranteed to
 eventually reach the recipient, provided that the recipient exists.
 
 The implementation details are outlined below:
 
-* Messages are sent via the :func:`pulsar.send` function, which is a proxy for
-  the :class:`pulsar.Actor.send` method. Here is how you ping actor ``abc``::
+* Messages are sent via the :func:`.send` function, which is a proxy for
+  the actor :meth:`~.Actor.send` method.
+  Here is how you ping actor ``abc`` in a coroutine::
 
       from pulsar import send
 
-      send('abc', 'ping')
+      def example():
+          result = yield send('abc', 'ping')
 
-* The :class:`pulsar.Arbiter` mailbox is a TCP :class:`pulsar.Server`
-  accepting :class:`pulsar.Connection` from remote actors.
-* The :attr:`pulsar.Actor.mailbox` is a :class:`pulsar.Client` of the arbiter
+* The :class:`.Arbiter` :attr:`~pulsar.Actor.mailbox` is a :class:`.TcpServer`
+  accepting connections from remote actors.
+* The :attr:`.Actor.mailbox` is a :class:`.MailboxClient` of the arbiter
   mailbox server.
 * When an actor sends a message to another actor, the arbiter mailbox behaves
   as a proxy server by routing the message to the targeted actor.
 * Communication is bidirectional and there is **only one connection** between
   the arbiter and any given actor.
 * Messages are encoded and decoded using the unmasked websocket protocol
-  implemented in :class:`pulsar.utils.websocket.FrameParser`.
+  implemented in :func:`.frame_parser`.
 * If, for some reasons, the connection between an actor and the arbiter
   get broken, the actor will eventually stop running and garbaged collected.
 
 
-Actor mailbox protocol
+Implementation
 =========================
   For the curious this is how the internal protocol is implemented
 
-  .. autoclass:: MailboxConsumer
-   :members:
-   :member-order: bysource
+Protocol
+~~~~~~~~~~~~
+
+.. autoclass:: MailboxProtocol
+  :members:
+  :member-order: bysource
+
+Client
+~~~~~~~~~~~~
+
+.. autoclass:: MailboxClient
+  :members:
+  :member-order: bysource
+
 '''
 import sys
 import logging
+import socket
+import pickle
 from collections import namedtuple
 
 from pulsar import ProtocolError, CommandError
-from pulsar.utils.pep import pickle
 from pulsar.utils.internet import nice_address
-from pulsar.utils.websocket import FrameParser
+from pulsar.utils.websocket import frame_parser
 from pulsar.utils.security import gen_unique_id
 
 from .access import get_actor
-from .defer import Failure, Deferred, maybe_async
-from .protocols import ProtocolConsumer
-from .clients import Client, Request
+from .futures import Future, coroutine_return, task
 from .proxy import actorid, get_proxy, get_command, ActorProxy
+from .protocols import Protocol
+from .clients import AbstractClient
 
 
 LOGGER = logging.getLogger('pulsar.mailbox')
@@ -93,11 +107,10 @@ class ProxyMailbox(object):
         pass
 
 
-class Message(Request):
+class Message(object):
     '''A message which travels from actor to actor.
     '''
-    def __init__(self, data, future=None, address=None, timeout=None):
-        super(Message, self).__init__(address, timeout)
+    def __init__(self, data, future=None):
         self.data = data
         self.future = future
 
@@ -106,8 +119,7 @@ class Message(Request):
     __str__ = __repr__
 
     @classmethod
-    def command(cls, command, sender, target, args, kwargs, address=None,
-                timeout=None):
+    def command(cls, command, sender, target, args, kwargs):
         command = get_command(command)
         data = {'command': command.__name__,
                 'sender': actorid(sender),
@@ -115,11 +127,11 @@ class Message(Request):
                 'args': args if args is not None else (),
                 'kwargs': kwargs if kwargs is not None else {}}
         if command.ack:
-            future = Deferred()
+            future = Future()
             data['ack'] = gen_unique_id()[:8]
         else:
             future = None
-        return cls(data, future, address, timeout)
+        return cls(data, future)
 
     @classmethod
     def callback(cls, result, ack):
@@ -127,26 +139,25 @@ class Message(Request):
         return cls(data)
 
 
-class MailboxConsumer(ProtocolConsumer):
-    '''The :class:`pulsar.ProtocolConsumer` for internal message passing
-between actors. Encoding and decoding uses the unmasked websocket
-protocol.'''
-    def connection_made(self, connection):
+class MailboxProtocol(Protocol):
+    '''The :class:`.Protocol` for internal message passing between actors.
+
+    Encoding and decoding uses the unmasked websocket protocol.
+    '''
+    def __init__(self, **kw):
+        super(MailboxProtocol, self).__init__(**kw)
         self._pending_responses = {}
-        self._parser = FrameParser(kind=2)
+        self._parser = frame_parser(kind=2, pyparser=True)
         actor = get_actor()
         if actor.is_arbiter():
-            self.connection.bind_event('connection_lost', None,
-                                       self._connection_lost)
+            self.bind_event('connection_lost', self._connection_lost)
 
     def request(self, command, sender, target, args, kwargs):
         '''Used by the server to send messages to the client.'''
         req = Message.command(command, sender, target, args, kwargs)
-        self.start_request(req)
+        self._start(req)
         return req.future
 
-    #######################################################################
-    ##    PROTOCOL CONSUMER IMPLEMENTATION
     def data_received(self, data):
         # Feed data into the parser
         msg = self._parser.decode(data)
@@ -155,45 +166,46 @@ protocol.'''
                 message = pickle.loads(msg.body)
             except Exception as e:
                 raise ProtocolError('Could not decode message body: %s' % e)
-            maybe_async(self._responde(message), event_loop=self.event_loop)
+            self._on_message(message)
             msg = self._parser.decode()
 
-    def start_request(self, req=None):
-        if req:
-            if req.future and 'ack' in req.data:
-                self._pending_responses[req.data['ack']] = req.future
-                try:
-                    self._write(req)
-                except Exception as e:
-                    req.future.callback(e)
-            else:
-                self._write(req)
-    start = start_request
-
     ########################################################################
     ##    INTERNALS
-    def _connection_lost(self, failure):
-        actor = get_actor()
-        if actor.is_running():
-            failure.log(msg='Connection lost with actor.', level='warning')
+    def _start(self, req):
+        if req.future and 'ack' in req.data:
+            self._pending_responses[req.data['ack']] = req.future
+            try:
+                self._write(req)
+            except Exception as exc:
+                req.future.set_exception(exc)
         else:
-            failure.mute()
-        return failure
+            self._write(req)
 
-    def _responde(self, message):
+    def _connection_lost(self, _, exc=None):
+        if exc:
+            actor = get_actor()
+            if actor.is_running():
+                actor.logger.warning('Connection lost with actor.')
+
+    @task
+    def _on_message(self, message):
         actor = get_actor()
         command = message.get('command')
-        #actor.logger.debug('handling %s', command)
+        ack = message.get('ack')
         if command == 'callback':
-            # this is a callback
-            self._callback(message.get('ack'), message.get('result'))
+            if not ack:
+                raise ProtocolError('A callback without id')
+            try:
+                pending = self._pending_responses.pop(ack)
+            except KeyError:
+                raise KeyError('Callback %s not in pending callbacks' % ack)
+            pending.set_result(message.get('result'))
         else:
             try:
                 target = actor.get_actor(message['target'])
                 if target is None:
-                    raise CommandError('Cannot execute "%s" in %s. Unknown '
-                                       'actor %s' % (command, actor,
-                                                     message['target']))
+                    raise CommandError('cannot execute "%s", unknown actor '
+                                       '"%s"' % (command, message['target']))
                 # Get the caller proxy without throwing
                 caller = get_proxy(actor.get_actor(message['sender']),
                                    safe=True)
@@ -209,57 +221,72 @@ protocol.'''
                 else:
                     actor = target
                     command = get_command(command)
-                    req = CommandRequest(target, caller, self.connection)
+                    req = CommandRequest(target, caller, self)
                     result = yield command(req, message['args'],
                                            message['kwargs'])
-            except Exception:
-                result = Failure(sys.exc_info())
-            if message.get('ack'):
-                req = Message.callback(result, message['ack'])
-                self.start_request(req)
-
-    def _callback(self, ack, result):
-        if not ack:
-            raise ProtocolError('A callback without id')
-        try:
-            pending = self._pending_responses.pop(ack)
-        except KeyError:
-            raise KeyError('Callback %s not in pending callbacks' % ack)
-        pending.callback(result)
+            except CommandError as exc:
+                self.logger.warning('Command error: %s' % exc)
+                result = None
+            except Exception as exc:
+                self.logger.exception('Unhandled exception')
+                result = None
+            if ack:
+                self._start(Message.callback(result, ack))
 
     def _write(self, req):
         obj = pickle.dumps(req.data, protocol=2)
-        data = self._parser.encode(obj, opcode=0x2).msg
+        data = self._parser.encode(obj, opcode=2)
         try:
-            self.transport.write(data)
-        except IOError:
+            self._transport.write(data)
+        except socket.error:
             actor = get_actor()
             if actor.is_running():
-                raise
+                if actor.is_arbiter():
+                    raise
+                else:
+                    actor.logger.warning('Lost connection with arbiter')
+                    actor._loop.stop()
 
 
-class MailboxClient(Client):
-    # mailbox for actors client
-    consumer_factory = MailboxConsumer
-    max_reconnect = 0
+class MailboxClient(AbstractClient):
+    '''Used by actors to send messages to other actors via the arbiter.
+    '''
+    protocol_factory = MailboxProtocol
 
-    def __init__(self, address, actor, event_loop):
-        super(MailboxClient, self).__init__(event_loop=event_loop)
+    def __init__(self, address, actor, loop):
+        super(MailboxClient, self).__init__(loop)
         self.address = address
-        self._consumer = None
         self.name = 'Mailbox for %s' % actor
+        self._connection = None
 
     def response(self, request):
         resp = super(MailboxClient, self).response
         self._consumer = resp(request, self._consumer, False)
         return self._consumer
 
+    def connect(self):
+        return self.create_connection(self.address)
+
     def __repr__(self):
         return '%s %s' % (self.name, nice_address(self.address))
 
+    @task
     def request(self, command, sender, target, args, kwargs):
         # the request method
-        req = Message.command(command, sender, target, args, kwargs,
-                              self.address, self.timeout)
-        self.response(req)
-        return req.future
+        if self._connection is None:
+            self._connection = yield self.connect()
+            self._connection.bind_event('connection_lost', self._lost)
+        req = Message.command(command, sender, target, args, kwargs)
+        self._connection._start(req)
+        response = yield req.future
+        coroutine_return(response)
+
+    def start_serving(self):
+        pass
+
+    def close(self):
+        if self._connection:
+            self._connection.close()
+
+    def _lost(self, _, exc=None):
+        self._loop.stop()
