@@ -1,12 +1,13 @@
 import sys
 from time import time
 
-import pulsar
+from pulsar import Config
 from pulsar.utils.pep import iteritems, itervalues, range
+from pulsar.utils.security import gen_unique_id
 
-from . import proxy
+from .proxy import actor_proxy_future
 from .actor import Actor
-from .defer import async_while
+from .futures import async_while
 from .concurrency import concurrency
 from .consts import *
 
@@ -22,20 +23,29 @@ def _spawn_actor(cls, monitor, cfg=None, name=None, aid=None, **kw):
     kind = None
     if issubclass(cls, PoolMixin):
         kind = 'monitor'
-    if cfg is None:
-        if monitor:
-            cfg = monitor.cfg.copy()
-        else:
-            cfg = pulsar.Config()
     if monitor:
         params = monitor.actorparams()
         name = params.pop('name', name)
         aid = params.pop('aid', aid)
-    else:  # monitor not available, this is the arbiter
+        cfg = params.pop('cfg', cfg)
+
+    # get config if not available
+    if cfg is None:
+        if monitor:
+            cfg = monitor.cfg.copy()
+        else:
+            cfg = Config()
+
+    if not monitor:  # monitor not available, this is the arbiter
         if kind != 'monitor':
             raise TypeError('class %s not a valid monitor' % cls)
         kind = 'arbiter'
         params = {}
+        if not cfg.exc_id:
+            if not aid:
+                aid = gen_unique_id()[:8]
+            cfg.set('exc_id', aid)
+    #
     for key, value in iteritems(kw):
         if key in cfg.settings:
             cfg.set(key, value)
@@ -58,32 +68,33 @@ def _spawn_actor(cls, monitor, cfg=None, name=None, aid=None, **kw):
     else:
         actor_proxy.monitor = monitor
         monitor.managed_actors[actor_proxy.aid] = actor_proxy
-        deferred = proxy.ActorProxyDeferred(actor_proxy)
+        future = actor_proxy_future(actor_proxy)
         actor_proxy.start()
-        return deferred
+        return future
 
 
 class PoolMixin(Actor):
-    '''Not an actor per se, this is a mixin for :class:`Actor`
-which manages a pool (group) of actors. Given an :attr:`actor_class`
-it makes sure there are always :attr:`cfg.workers` alive.
-It is used by both the :class:`Arbiter` and the :class:`Monitor` classes.
+    '''A mixin for :class:`.Actor` which manages a pool (group) of actors.
 
-.. attribute:: managed_actors
+    Given an :attr:`actor_class` it makes sure there are always
+    :attr:`cfg.workers` alive.
+    It is used by both the :class:`Arbiter` and the :class:`Monitor` classes.
 
-    dictionary with keys given by actor's ids and values by
-    :class:`ActorProxyMonitor` instances. These are the actors managed by the
-    pool.
+    .. attribute:: managed_actors
 
-.. attribute:: terminated_actors
+        dictionary with keys given by actor's ids and values by
+        :class:`.ActorProxyMonitor` instances. These are the actors
+        managed by the pool.
 
-    list of :class:`ActorProxyMonitor` which have been terminated
-    (the remote actor did not have a cleaned shutdown).
-'''
+    .. attribute:: terminated_actors
+
+        list of :class:`.ActorProxyMonitor` which have been terminated
+        (the remote actor did not have a cleaned shutdown).
+    '''
     CLOSE_TIMEOUT = 30000000000000
     actor_class = Actor
     '''The class derived form :class:`Actor` which the monitor manages
-during its life time.
+    during its life time.
 
     Default: :class:`Actor`'''
 
@@ -91,7 +102,6 @@ during its life time.
         super(PoolMixin, self).__init__(impl)
         self.managed_actors = {}
         self.terminated_actors = []
-        self.actor_class = self.params.pop('actor_class') or self.actor_class
 
     def get_actor(self, aid):
         aid = getattr(aid, 'aid', aid)
@@ -104,7 +114,8 @@ during its life time.
 
     def spawn(self, actor_class=None, **params):
         '''Spawn a new :class:`Actor` and return its
-:class:`ActorProxyMonitor`.'''
+        :class:`.ActorProxyMonitor`.
+        '''
         actor_class = actor_class or self.actor_class
         return _spawn_actor(actor_class, self, **params)
 
@@ -114,23 +125,26 @@ during its life time.
         The disctionary is passed to the spawn method when creating new
         actors. Fire the :ref:`on_params actor hook <actor-hooks>`.
         '''
-        data = dict(self.params)
+        data = {}
         self.fire_event('on_params', params=data)
         return data
 
     def _remove_actor(self, actor, log=True):
-        if log:
-            self.logger.info('Removing %s', actor)
-        self.managed_actors.pop(actor.aid, None)
+        removed = self.managed_actors.pop(actor.aid, None)
+        if log and removed:
+            log = False
+            self.logger.warning('Removing %s', actor)
         if self.monitor:
-            self.monitor._remove_actor(actor, False)
+            self.monitor._remove_actor(actor, log)
+        return removed
 
     def manage_actors(self, stop=False):
         '''Remove :class:`Actor` which are not alive from the
-:class:`PoolMixin.managed_actors` and return the number of actors still alive.
+        :class:`PoolMixin.managed_actors` and return the number of actors
+        still alive.
 
-:parameter stop: if ``True`` stops all alive actor.
-'''
+        :parameter stop: if ``True`` stops all alive actor.
+        '''
         alive = 0
         if self.managed_actors:
             for aid, actor in list(iteritems(self.managed_actors)):
@@ -139,12 +153,12 @@ during its life time.
 
     def manage_actor(self, actor, stop=False):
         '''If an actor failed to notify itself to the arbiter for more than
-the timeout, stop the actor.
+        the timeout, stop the actor.
 
-:param actor: the :class:`Actor` to manage.
-:param stop: if ``True``, stop the actor.
-:return: if the actor is alive 0 if it is not.
-'''
+        :param actor: the :class:`Actor` to manage.
+        :param stop: if ``True``, stop the actor.
+        :return: if the actor is alive 0 if it is not.
+        '''
         if not self.is_running():
             stop = True
         if not actor.is_alive():
@@ -190,8 +204,8 @@ do nothing.'''
                 self.spawn()
 
     def stop_actors(self):
-        """Maintain the number of workers by spawning or killing
-as required."""
+        """Maintain the number of workers by spawning or killing as required
+        """
         if self.cfg.workers:
             num_to_kill = len(self.managed_actors) - self.cfg.workers
             for i in range(num_to_kill, 0, -1):
@@ -208,21 +222,21 @@ as required."""
 
 
 class Monitor(PoolMixin):
-    '''A monitor is a **very** special :class:`Actor`.
+    '''A monitor is a **very** special :class:`.Actor`.
 
-    it is a :class:`PoolMixin` which shares the same :class:`EventLoop`
-    with the :class:`Arbiter` and therefore lives in the main thread
+    it is a :class:`.PoolMixin` which shares the same event loop
+    with the :class:`.Arbiter` and therefore lives in the main thread
     of the master process domain.
 
-    The Arbiter manages monitors which in turn manage a set of :class:`Actor`
+    The Arbiter manages monitors which in turn manage a set of :class:`.Actor`
     performing similar tasks.
 
     In other words, you may have a monitor managing actors for serving HTTP
     requests on a given port, another monitor managing actors consuming tasks
-    from a task queue and so forth. You can think of :class:`Monitor` as
-    managers of pools of :class:`Actor`.
+    from a task queue and so forth. You can think of :class:`.Monitor` as
+    managers of pools of :class:`.Actor`.
 
-    Monitors are created by invoking the :meth:`Arbiter.add_monitor`
+    Monitors are created by invoking the :meth:`.Arbiter.add_monitor`
     functions and not by directly invoking the constructor. Therefore
     adding a new monitor to the arbiter follows the pattern::
 
@@ -238,8 +252,9 @@ class Monitor(PoolMixin):
         return True
 
     def monitor_task(self):
-        '''Monitor specific task called by the :meth:`Monitor.periodic_task`.
+        '''Monitor specific task.
 
+        Called by the :meth:`.MonitorConcurrency.periodic_task` method.
         By default it does nothing. Override if you need to.
         '''
         pass
